@@ -116,6 +116,20 @@ def interceptor(rn_environ, *args, **kwargs):
         errors = [callback.reason_phrase, callback.__class__.__name__]
         interceptor_error(status_code, errors)
     
+    if conf.profile_http_header_enabled:
+        text = ''
+        items = ['host', 'connection','cache_control', 'user_agent',
+                'upgrade_insecure_requests',
+                'accept', 'accept_encoding', 'accept_language',
+                'cookie']
+        for key in items:
+            if environ.get('HTTP_{}'.format(key.upper())):
+                text += '{}={}\n'.format(key, environ.get('HTTP_{}'.format(key.upper())))
+
+        datas = [' ', 'HTTP-HEADERS', text]
+        ctx.start_time = DateUtil.nowSystem()
+        UdpSession.send_packet(PacketTypeEnum.PACKET_MSG, ctx, datas)
+
     end_interceptor()
     return callback
 
@@ -236,40 +250,57 @@ def interceptor_db_close(fn, *args, **kwargs):
     return callback
 
 
-# timeunit = 0
-# httpc_on = 0
-# httpc_off = 0
-
+check_seq = 1
 def inter_tx_trace_auto_on(ctx):
-    if int(conf.mtrace_rate) <= 0 or ctx.httpc_checked or ctx.mtid != 0:
-        return
-    
-    ctx.httpc_checked = True
-    ctx.mtid = TraceContextManager.getId()
-    
-    #
-    # tu = DateUtil.currentTime() / DateUtil.MILLIS_PER_FIVE_MINUTE
-    # if tu != timeunit:
-    #     timeunit = tu
-    #     httpc_on = 1
-    #     httpc_off = 0
-    #     ctx.mtid = TraceContextManager.getId()
-    #     return
-    #
-    # on_off_tot = httpc_on + httpc_off
-    # if on_off_tot <= 0:
-    #     ctx.mtid = TraceContextManager.getId()
-    #     httpc_on = 1
-    #     httpc_off = 0
-    #     return
-    # cr = (httpc_on * 100) / on_off_tot
-    # if cr <= conf.mtrace_rate:
-    #     ctx.mtid = TraceContextManager.getId()
-    #     httpc_on=+1
-    # else:
-    #     httpc_off=+1
-    
-    
+    try:
+        if isinstance(conf.mtrace_rate, str):
+            conf.mtrace_rate = int(conf.mtrace_rate)
+    except ValueError:
+        conf.mtrace_rate = 0
+    finally:
+        if conf.mtrace_rate <= 0 or ctx.httpc_checked or ctx.mtid != 0:
+            return
+        
+        ctx.httpc_checked = True
+        
+        try:
+            inter_tx_trace_auto_on.check_seq += 1
+        except AttributeError:
+            inter_tx_trace_auto_on.check_seq = 1
+        finally:
+            check_seq = inter_tx_trace_auto_on.check_seq
+            
+            rate = int(conf.mtrace_rate / 10)
+            if rate == 10:
+                ctx.mtid = TraceContextManager.getId()
+            elif rate == 9:
+                if check_seq % 10 != 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 8:
+                if check_seq % 5 != 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 7:
+                if check_seq % 4 != 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 6:
+                if check_seq % 3 != 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 5:
+                if check_seq % 2 == 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 4:
+                if check_seq % 3 == 0 or check_seq % 5 == 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 3:
+                if check_seq % 4 == 0 or check_seq % 5 == 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 2:
+                if check_seq % 5 == 0:
+                    ctx.mtid = TraceContextManager.getId()
+            elif rate == 1:
+                if check_seq % 10 == 0:
+                    ctx.mtid = TraceContextManager.getId()
+                    
 def transfer(ctx, headers):
     if not ctx.mtid:
         inter_tx_trace_auto_on(ctx)
